#include <iostream> 
#include <random>

#include "Vector.h"

template<typename T = double>
struct RadianDegree
{
    T value = 0;
    RadianDegree() = default;
    RadianDegree(T value)
        : value(value)
    {
    }
};

template <typename T>
std::ostream& operator<<(std::ostream& os, const RadianDegree<T>& radian_degree)
{
    os << radian_degree.value;
    return os;
}

void test_1()
{
    // domyslna polityka sprawdzania = no_checking_policy
    auto v = Vector<RadianDegree<>>();

    for (int i = 0; i < 10; ++i)
    {
        v.push_back(RadianDegree<>(i));
    }

    v[6].value = 20;
    // v[20].value = 5; // niezdefiniowane dzialanie (ang. undefined behaviour)

    for (auto& radian_degree : v)
    {
        std::cout << radian_degree << " ";
    }
}

void test_2()
{
    auto v = Vector<RadianDegree<>, assert_policy>();

    for (int i = 0; i < 10; ++i)
    {
        v.push_back(RadianDegree<>(i));
    }

    v[6].value = 20;
    // v[20].value = 5; // assertion failed 

    for (auto& radian_degree : v)
    {
        std::cout << radian_degree << " ";
    }
}

void test_3()
{
    auto v = Vector<RadianDegree<>, resize_policy>();

    for (int i = 0; i < 10; ++i)
    {
        v.push_back(RadianDegree<>(i));
    }

    v[6].value = 20;
    v[20].value = 5; 

    for (auto& radian_degree : v)
    {
        std::cout << radian_degree << " ";
    }
}

int main(void)
{
    test_1();
    std::cout << std::endl;
    test_2();
    std::cout << std::endl;
    test_3();
    std::cout << std::endl;

    getchar();
    return 0;
}
